﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Telerik.Windows;
using Telerik.Windows.Controls.DragDrop;
using System.Collections;
using System.Collections.ObjectModel;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.TreeView;
using Telerik.Windows.Controls.GridView;
using System.Windows.Media.Imaging;

namespace DragDropExamplesImprovement
{
	public partial class MainPage : UserControl
	{
		public MainPage()
		{
			InitializeComponent();

			// Set the items source of the controls:
			allProductsView.ItemsSource = CategoryViewModel.Generate();

			var orderSource = new ObservableCollection<ProductViewModel>();
			foreach (var product in ProductViewModel.Generate(4))
			{
				orderSource.Add(product);
			}
			orderView.ItemsSource = orderSource;

			var wishlistSource = new ObservableCollection<ProductViewModel>();
			foreach (var product in ProductViewModel.Generate(6))
			{
				wishlistSource.Add(product);
			}
			wishlistView.ItemsSource = wishlistSource;

			// Allow dropping into the ListBox and GridView only if the
			// dragged items are all products:
			RadDragAndDropManager.AddDropQueryHandler(wishlistView, OnDropQuery);
			RadDragAndDropManager.AddDropQueryHandler(orderView, OnDropQuery);

			// Change the drag cue and choose an action for the sucessful drop
			// in the Wishlist:
			RadDragAndDropManager.AddDropInfoHandler(wishlistView, OnWishlistDropInfo);

			// Change the drag cue and choose an action for the sucessful drop
			// in the Order GridView:
			RadDragAndDropManager.AddDropInfoHandler(orderView, OnGridViewDropInfo);

			// Allow dragging of the Wishlist and Order items:
			RadDragAndDropManager.AddDragQueryHandler(wishlistView, OnWishListDragQuery);
			RadDragAndDropManager.AddDragQueryHandler(orderView, OnOrderDragQuery);

			// Handle the case when items are dragged away from  the ListBox
			// and the Order:
			RadDragAndDropManager.AddDragInfoHandler(wishlistView, OnWishListDragInfo);
			RadDragAndDropManager.AddDragInfoHandler(orderView, OnOrderDragInfo);

			//Only drop before:
			allProductsView.PreviewDragEnded += new RadTreeViewDragEndedEventHandler(allProductsView_PreviewDragEnded);
		}

		void allProductsView_PreviewDragEnded(object sender, RadTreeViewDragEndedEventArgs e)
		{
			var treeViewItemDestination = e.TargetDropItem;
			//We want to cancel removing the item only if it is dropped outside the current TreeView.
			if (treeViewItemDestination == null || treeViewItemDestination.ParentTreeView != sender)
			{
				e.Handled = true;
			}
		}

		private void OnOrderDragInfo(object sender, DragDropEventArgs e)
		{
			var gridView = sender as RadGridView;
			var draggedItems = e.Options.Payload as IEnumerable;

			if (e.Options.Status == DragStatus.DragInProgress)
			{
				//Set up a drag cue:
				var cue = new TreeViewDragCue();
				//Here we need to choose a template for the items:
				cue.ItemTemplate = this.Resources["ProductTemplate"] as DataTemplate;
				cue.ItemsSource = draggedItems;
				e.Options.DragCue = cue;
			}
			else if (e.Options.Status == DragStatus.DragComplete)
			{
				var source = gridView.ItemsSource as IList;
				foreach (object draggedItem in draggedItems)
				{
					source.Remove(draggedItem);
				}
			}
		}

		private void OnWishListDragInfo(object sender, DragDropEventArgs e)
		{
			var listBox = sender as System.Windows.Controls.ListBox;
			var draggedItems = e.Options.Payload as IEnumerable;

			if (e.Options.Status == DragStatus.DragInProgress)
			{
				//Set up a drag cue:
				var cue = new TreeViewDragCue();
				cue.ItemTemplate = listBox.ItemTemplate;
				cue.ItemsSource = draggedItems;
				e.Options.DragCue = cue;
			}
			else if (e.Options.Status == DragStatus.DragComplete)
			{
				var source = listBox.ItemsSource as IList;
				foreach (object draggedItem in draggedItems)
				{
					source.Remove(draggedItem);
				}
			}
		}

		private void OnOrderDragQuery(object sender, DragDropQueryEventArgs e)
		{
			var gridView = sender as RadGridView;
			if (gridView != null)
			{
				var selectedItems = gridView.SelectedItems.ToList();
				e.QueryResult = selectedItems.Count > 0;
				e.Options.Payload = selectedItems;
			}

			e.QueryResult = true;
			e.Handled = true;
		}

		private void OnWishListDragQuery(object sender, DragDropQueryEventArgs e)
		{
			var listBox = sender as System.Windows.Controls.ListBox;
			if (listBox != null)
			{
				var selectedItems = listBox.SelectedItems.Cast<object>().ToList();
				e.Options.Payload = selectedItems;
			}

			e.QueryResult = true;
			e.Handled = true;
		}

		private void OnGridViewDropInfo(object sender, DragDropEventArgs e)
		{
			var order = e.Options.Destination as RadGridView;
			var draggedItems = e.Options.Payload as ICollection;

			// Get the drag cu that the TreeView or we have created
			var cue = e.Options.DragCue as TreeViewDragCue;

			if (e.Options.Status == DragStatus.DropPossible)
			{
				// Set a suitable text:
				cue.DragActionContent = String.Format("Add {0} item{1} to Order", draggedItems.Count, draggedItems.Count > 1 ? "s" : String.Empty);
				cue.IsDropPossible = true;
				order.Background = this.Resources["DropPossibleBackground"] as Brush;
			}
			else if (e.Options.Status == DragStatus.DropImpossible)
			{
				cue.DragActionContent = null;
				cue.IsDropPossible = false;
			}
			else if (e.Options.Status == DragStatus.DropComplete)
			{
				var items = order.ItemsSource as IList;
				foreach (var draggedItem in draggedItems)
				{
					items.Add(draggedItem);
				}
			}

			if (e.Options.Status != DragStatus.DropPossible)
			{
				order.Background = new SolidColorBrush(Colors.White);
			}
		}

		private void OnWishlistDropInfo(object sender, DragDropEventArgs e)
		{
			var wishlist = e.Options.Destination as System.Windows.Controls.ItemsControl;
			var draggedItems = e.Options.Payload as ICollection;

			// Get the drag cu that the TreeView or we have created
			var cue = e.Options.DragCue as TreeViewDragCue;

			if (e.Options.Status == DragStatus.DropPossible)
			{
				// Set a suitable text:
				cue.DragActionContent = String.Format("Add {0} item{1} to Wishlist", draggedItems.Count, draggedItems.Count > 1 ? "s" : String.Empty);
				cue.IsDropPossible = true;
				wishlist.Background = this.Resources["DropPossibleBackground"] as Brush;
			}
			else if (e.Options.Status == DragStatus.DropImpossible)
			{
				cue.DragActionContent = null;
				cue.IsDropPossible = false;
			}
			else if (e.Options.Status == DragStatus.DropComplete)
			{
				var items = wishlist.ItemsSource as IList;
				foreach (var draggedItem in draggedItems)
				{
					items.Add(draggedItem);
				}
			}

			if (e.Options.Status != DragStatus.DropPossible)
			{
				wishlist.Background = new SolidColorBrush(Colors.White);
			}
		}

		private void OnDropQuery(object sender, DragDropQueryEventArgs e)
		{
			// We allow drop only if the dragged items are products:
			var draggedItems = e.Options.Payload as ICollection;
			var result = draggedItems.Cast<object>().All(item => item is ProductViewModel);
			e.QueryResult = result;
			e.Handled = true;

			// Note that here we agree to accept a drop. We will be notified
			// in the DropInfo event whether a drop is actually possible.
		}
	}

	public class ExampleTemplateSelector : DataTemplateSelector
	{
		public override DataTemplate SelectTemplate(object item, DependencyObject container)
		{
			if (item is ProductViewModel)
			{
				return ProductTemplate;
			}
			else if (item is CategoryViewModel)
			{
				return CategoryTemplate;
			}
			return null;
		}

		public DataTemplate ProductTemplate { get; set; }
		public DataTemplate CategoryTemplate { get; set; }
	}

	public class CategoryViewModel
	{
		public static IList Generate()
		{
			var latest = new CategoryViewModel() { Title = "Latest Additions" };
			foreach (var item in ProductViewModel.Generate(4))
			{
				latest.Items.Add(item);
			}

			var highestRated = new CategoryViewModel() { Title = "Highest Rated" };
			foreach (var item in ProductViewModel.Generate(5))
			{
				highestRated.Items.Add(item);
			}

			var onSale = new CategoryViewModel() { Title = "On Sale" };
			var onSale2 = new CategoryViewModel() { Title = "On Sale 2" };
			var onSale3 = new CategoryViewModel() { Title = "On Sale 3" };
			

			var value = new CategoryViewModel() { Title = "Instant Deal" };
			value.Items.Add(onSale);
			value.Items.Add(onSale2);
			value.Items.Add(onSale3);

			var child1 = new CategoryViewModel() { Title = "Child 1" };
			var child2 = new CategoryViewModel() { Title = "Child 2" };
			var child3 = new CategoryViewModel() { Title = "Child 3" };

			onSale2.Items.Add(child1);
			onSale2.Items.Add(child2);
			onSale2.Items.Add(child3);

			var gchild1 = new CategoryViewModel() { Title = "Grandchild 1" };
			var gchild2 = new CategoryViewModel() { Title = "Grandchild 2" };
			var gchild3 = new CategoryViewModel() { Title = "Grandchild 3" };

			child3.Items.Add(gchild1);
			child3.Items.Add(gchild2);
			child3.Items.Add(gchild3);
			

			return new ObservableCollection<object>()
			{
				latest, highestRated, value 
			};


		}

		public CategoryViewModel()
		{
			Items = new ObservableCollection<object>();
		}

		public string Title { get; set; }
		public IList Items { get; set; }
	}

	public class ProductViewModel
	{
		// Data generation.
		private static Random generator = new Random(567665846);
		private static string[] adjectives = "Fabulous,Amazing,New,Classic,Modern,Durable,Outstanding,Excellent,Premium".Split(',');
		private static string[] owner = "Alaska,Jonhn,Ray,Ruby,Stone,Lilly,Scott,Barney,Dorian,Neo,Sarah".Split(',');
		private static string[] objects = "Pen,Manual,Bicycle,Umbrella,Mouse,Vase,Keyboard".Split(',');
		private static decimal[] prices = { 12.99M, 13.15M, 24.99M, 33.99M, 9.99M, 15.99M, 16.99M, 12.50M };

		public static IEnumerable<ProductViewModel> Generate(int count)
		{
			for (int i = 0; i < count; i++)
			{
				var product = objects[generator.Next(objects.Length)];
				yield return new ProductViewModel()
				{
					Name = String.Format("{0}'s {1} {2}",
						owner[generator.Next(owner.Length)],
						adjectives[generator.Next(adjectives.Length)],
						product),
					UnitPrice = prices[generator.Next(prices.Length)],
					Description = String.Format("Exquisite handcrafted {0}.", product.ToLower())
				};
			}
		}

		public string Name { get; set; }
		public string Description { get; set; }
		public decimal UnitPrice { get; set; }
	}
}
